<?php

/**
 * @file Contains our drush command to load and unload fixtures
 *
 */

namespace Drupal\data_fixtures\Commands;

use Drupal\data_fixtures\FixturesManager;
use Drush\Commands\DrushCommands;

/**
 *  Drush ^9 commands for data_fixtures module.
 */
class DataFixturesCommands extends DrushCommands {

  /**
   * @var \Drupal\data_fixtures\FixturesManager
   */
  protected $fixturesManager;

  /**
   * DataFixturesCommands constructor.
   *
   * @param \Drupal\data_fixtures\FixturesManager $fixturesManager
   */
  public function __construct(FixturesManager $fixturesManager)
  {
    $this->fixturesManager = $fixturesManager;
  }

  /**
   * List all enabled fixtures that are currently installed
   *
   * @command fixtures:list
   * @aliases fixtures-list
   *
   * @validate-module-enabled data_fixtures
   *
   */
  public function listFixtures() {
    $this->output()->writeln('Listing all fixtures:');
    foreach ($this->data_fixtures_get_fixtures('all') as $generator) {
      $this->output()->writeln(' - ' . $generator->prettyPrint());
    }
  }

  /**
   * Load all enabled fixtures that are currently installed
   *
   * @param $fixture_alias
   *   The alias of the fixture to load. "all" if you want to load all fixtures
   *
   * @command fixtures:load
   * @aliases fixtures-load
   *
   * @validate-module-enabled data_fixtures
   *
   */
  public function load($fixture_alias) {
    foreach ($this->data_fixtures_get_fixtures($fixture_alias) as $generator) {
      $this->output()->writeln('Loading: ' . $generator->prettyPrint());
      $generator->getGenerator()->load();
    }
  }

  /**
   * Unload all enabled fixtures that are currently installed.
   *
   * @param $fixture_alias
   *   The alias of the fixture to unload. "all" if you want to unload all fixtures
   *
   * @command fixtures:unload
   * @aliases fixtures-unload
   *
   * @validate-module-enabled data_fixtures
   *
   */
  public function unload($fixture_alias) {
    foreach ($this->data_fixtures_get_fixtures($fixture_alias, true) as $generator) {
      $this->output()->writeln('Unloading: ' . $generator->prettyPrint());
      $generator->getGenerator()->unLoad();
    }
  }

  /**
   * Reload (unload + load) all enabled fixtures that are currently installed
   *
   * @param $fixture_alias
   *   The alias of the fixture to reload. "all" if you want to reload all fixtures
   *
   * @command fixtures:reload
   * @aliases fixtures-reload
   *
   * @validate-module-enabled data_fixtures
   *
   */
  public function reload($fixture_alias) {
    $this->unload($fixture_alias);
    $this->load($fixture_alias);
  }

  /**
   * @param $fixtures_alias
   * @param bool $reverse
   *
   * @return array|\Drupal\data_fixtures\FixturesGenerator[]
   */
  private function data_fixtures_get_fixtures($fixtures_alias, $reverse = false) {
    if ('all' === $fixtures_alias) {
      return $this->fixturesManager->getGenerators($reverse);
    }

    return array_filter(
      $this->fixturesManager->getGenerators(),
      function (\Drupal\data_fixtures\FixturesGenerator $fixtures_generator) use ($fixtures_alias) {
        return $fixtures_generator->getAlias() === $fixtures_alias;
      }
    );
  }
}
